#version version;

//#version 3.7;

#declare SDL = true;
#declare Radiosity = off;
#declare Level = 0;		//0 = low, 1 = high

global_settings {
	assumed_gamma 1.0
	ambient_light rgb 0
	max_trace_level 7
	
	#if (Radiosity)
		#if (Level)
			// High quality settings
			radiosity {
				adc_bailout 0.005
				always_sample off
				count 150			// 35 - 1600
				error_bound 0.25
				gray_threshold 0.2
				low_error_factor 0.25
				minimum_reuse 0.01
				maximum_reuse 0.2
				nearest_count 8
				pretrace_start 0.08
				pretrace_end   0.04 	//  0.01
				recursion_limit 5
				//brightness 1
			}  // end radiosity block
		#else
			// Low quality settings
			radiosity {
				pretrace_start 1	//  0.08
				pretrace_end   1	//  0.04 	//  0.01
				count 35  		//  150
				nearest_count 5  	//  10
				error_bound 1.8		//  0.5
				recursion_limit 2
				low_error_factor 0.5
				gray_threshold 0.0
				minimum_reuse 0.015  	//  0.005
				maximum_reuse 0.2
				brightness 1
				adc_bailout 0.01   	//  0.005
			}  // end radiosity block
		#end
	#else
	
	#end
}

#include "colors.inc"
#include "math.inc"
#include "BillsArrayMacros.pov"
#include "Fmod.inc"

#default {
   // default textures
}

light_source {
	<5, 10, -20> 
	color White
	fade_distance 20
	fade_power 2
}



//#####################################################
#declare ParametricPoints = false;
#declare ParametricGrid = false;
#declare Corners = true;				// Corners of patches
#declare Grid = false;				// Bezier control grid
#declare UStep = 1;
#declare VStep = 1;
#declare ShowIVectors = false;		// full length Interpolation vectors
#declare D = 0;					// Noise 1-perfect for donut
#declare Squares = false;
#declare SquareTexture = texture {pigment {rgbt <0, 0, 0.2, 0.5>} finish {specular 0.01}}
#declare Splines = true;				// only generates Bezier splines using CORNER points
#declare Curve = true;
#declare CurveStep = 10;
#declare CurveTex = texture {pigment {rgb 1} finish {emission 1}}
#declare BT = 3;					// Bezier triangles in approximation
#declare Patches = true;
#declare UVTexture = 
	texture  { 
		pigment {image_map {png "GridMacro.png"}}  
		normal {bump_map {png "GridMacro.png" bump_size 2}} 
		finish {specular 0.1 reflection 0.05} 
		scale 1
		//translate x*0.3
		translate -y*0.3
	}

//#####################################################


camera { 
    //location  <0, 0, -20> rotate -y*95  //debug @ square 1
    //location  <0, 0, -30>
    location  <0, 2, -35>  //rotate -y*20
    look_at   <0, 0, -10>

	//location <-50, 10, -110>
	//look_at <-50, 10, -50>    

    right x*image_width/image_height
    up y
}



plane {y, -14 texture {
	pigment {
		checker
		Gray10 + Blue*0.1
		Gray10 + Red*0.1		
	} 
	finish {specular 0.2 reflection 0.7 }} 
}

/*
#if (version < 3.8)
	#ifndef (tau)
    #declare tau = 2*pi;
    #end
#end
*/

// macros and functions to stitch an arbitrary number of Bezier patches together
// into a plane
// into a tube
// into a torus

//------------------------------------------------------------------------------
// Fast NChooseM
#declare FastNCM = function (n, k) {prod(i, k+1, n, i) / prod(i, 1, n-k, i)}
//------------------------------------------------------------------------------
// Cycles through array elements backwards or forwards
// -1 of an array with dimension size N gives element N-1
// N of an array with dimension size N gives element 0
//#declare Cycle = function (_N, _Size) {select (_N, (_Size-1)+mod(_N+1, _Size), mod(_N, _Size) )}
//------------------------------------------------------------------------------
#macro Choose (_N, _Array)
	// returns an element from an array
	// uses mod () to cycle through array no matter the size of _N or _Array
	
	// *** Add optional dimension to access other than 1

	
	#local _Size = dimension_size (_Array,1);
	#local __N = select (_N, (_Size + _N), 0, _N);
	#local _Item = mod (__N, _Size);
	#local _Result = _Array [_Item];
	
	_Result
#end
//------------------------------------------------------------------------------
#declare Round = function (Value) {ceil(Value - 0.5)}
//------------------------------------------------------------------------------
#declare PowerFn = function(s, p) { select(p, pow(s, p), 1, pow(s, p)) }
//------------------------------------------------------------------------------
#declare Bernstein = function (Deg, K, S)
{FastNCM(Deg, K) * PowerFn(S, K) * PowerFn (1 - S, Deg - K)}

//------------------------------------------------------------------------------
#macro Point (VG, UG, Uc, Vc)			// called during interpolation loop due to using Current [V][U] array
	//#debug concat(Scalar(VG, 0, 0), ", ", Scalar(UG, 0, 1), "\n")
#local Degree = 3;
	#local P =
	#for (j, 0, 3)
		#for (i, 0, 3)
		//#debug concat(Scalar(VG, 0, 0), ", ", Scalar(UG, 0, 0), ", ", Scalar(i, 0, 0), ", ", Scalar(j, 0, 1), "\n")
		Bernstein(Degree, i, Vc) * Bernstein(Degree, j, Uc) * StitchedArray [VG][UG][i][j] +
		#end
	#end
	0;
	P
#end
//------------------------------------------------------------------------------
#macro Check (Vec1, Vec2, Color)
    #local TestV = VEq(Vec1, Vec2);
    #local Nudge = 1;
    #if (TestV)
        
    #else
        cone {Vec1*Nudge, Radius*3, Vec2*Nudge, Radius*1 texture {pigment {Color} finish {specular 0.2}} }
    #end
#end
//------------------------------------------------------------------------------
#macro Interpolate (Vec1, Vec2, Vec3)
	
	#local LengthA  = vlength (Vec1 - Vec3);
	#local LengthB  = vlength (Vec2 - Vec3);
	#local Length  = min (LengthA, LengthB);
	#local NVector = vnormalize (Vec1 - Vec2);
	#local _Vector  = (Length * NVector) / 3;
	_Vector
#end
//------------------------------------------------------------------------------
#macro ShowSquares (CP1, CP2, CP3, CP4, Color)
	triangle {CP1, CP2, CP3 texture {Color} }
	triangle {CP2, CP4, CP3 texture {Color} }
#end

//####################################################################################

// Use a set of parametric equations to calculate the corner points of a grid
// on a torus

#declare R1 = 10;			//
#declare R2 = 4;			//
#declare Sum = R1+R2;		//

#declare U1 = 0;			//	small radius
#declare U2 = 2*pi;			//
#declare V1 = 0;			//	large radius
#declare V2 = 2*pi;			//
#declare r0 = R1;
#declare r1 = R2;
//#declare Phi = U1;
#declare   Phi_inc = (2*pi/360)*60;	// increment Phi by 30 degrees (12 subdivisions) around small rad
#declare Theta_inc = (2*pi/360)*20;   // increment Theta by 10 degrees (24 subdivisions) around large rad
#declare Radius = 0.02; // radius for spheres and cylinders
#declare Spheres = true;
#declare Faces = true;
#declare Mesh = true;
#declare Colored = false;
#declare Spec = 0.0;
#declare Saturation = 0.8;

//###################################################################
// Create a set of points on the surface of a Torus
#declare X = function (T, P, R, r) {cos(T) * ( R + r * cos(P) )} 
#declare Y = function (T, P, R, r) {-sin(T) * ( R + r * cos(P) )} 
#declare Z = function (T, P, r, n) {r * sin(P)}
//------------------------------------------------------------------------------

// Spiral around a torus
#declare N = 5;
#declare XS = function (T, P, R, r) {(R + r * cos (N*T)) * cos (T)}
#declare YS = function (T, P, R, r) {(R + r * cos (N*T)) * sin (T)}
#declare ZS = function (T, P, R, r) { r * sin (N*T)}

//Trianguloid Trefoil
#declare Scale = 2;
//#declare X = function (T, P, N, n) { Scale * (2*sin(3*T) / (2 + cos(P)) ) }	// x-axis
//#declare Y = function (T, P, N, n) { Scale * (2*(sin(T) + 2*sin(2*T)) / (2 + cos(P + 2*pi / 3)) ) }	// y-axis
//#declare Z = function (T, P, N, n) { Scale * ( (cos(T) - 2*cos(2*T))*(2 + cos(P))*(2 + cos(P + 2*pi / 3)) / 4 ) }	// z-axis
//------------------------------------------------------------------------------

#debug "Creating arrays... \n"

// Create an array to hold all of the Bezier patches to cover the surface
#declare Usize = Round ((U2-U1)/Phi_inc)+1;
#declare Vsize = Round((V2-V1)/Theta_inc);
#debug concat ("Usize = ", str(Usize, 0, 3), ",   Vsize = ", str(Vsize, 0, 3), "\n")


#declare UU = Usize-1;
#declare VV = Vsize-1;

#debug concat ("UU = ", str(UU, 0, 3), ",   VV = ", str(VV, 0, 3), "\n")

#declare UArray = array [Usize+1];
#declare VArray = array [Vsize+1];

#debug concat ("UArray extends from 0 to ", str(dimension_size(UArray,1)-1, 0, 3), " \n")
#debug concat ("VArray extends from 0 to ", str(dimension_size(VArray,1)-1, 0, 3), " \n")

#declare StitchedArray = array[Vsize][Usize];

#for (U, 0, UU)
	#for (V, 0, VV)
		#local StitchedArray [V][U] = array[4][4];
	#end
	
#end

#debug concat ("StitchedArray is ", str(dimension_size(StitchedArray, 1)-1, 0, 3), " x ", str(dimension_size(StitchedArray, 2)-1, 0, 3), " elements \n\n")

//----------------------------------------------------


#declare Ustep = (U2-U1)/(Usize);		// Usize is the correct value to use
#declare Vstep = (V2-V1)/(Vsize);		// Vsize is the correct value to use

#debug concat ("UStep = U2 (", Scalar (U2, 5, 0), ") - U1 (", Scalar (U1, 5, 0), ") / Usize (", Scalar (Usize, 5, 0), ") = ", Scalar (Ustep, 5, 1))
#debug concat ("VStep = V2 (", Scalar (V2, 5, 0), ") - V1 (", Scalar (V1, 5, 0), ") / Vsize (", Scalar (Vsize, 5, 0), ") = ", Scalar (Vstep, 5, 1))

#debug "\n\n"

#for (U, 0, UU)     // must be UU because SitchedArray is only that big
	#declare UArray [U] = <U, max(0, U*Ustep), min((U+1)*Ustep, U2)>;
	//#debug concat ("UArray [", Scalar (U, 0, 0), "] = ", Vector (UArray [U], 5, 1))
#end

#for (V, 0, VV) // must be VV because SitchedArray is only that big
	#declare VArray [V] = <V, max(0, V*Vstep), min((V+1)*Vstep, V2)>;
	//#debug concat ("VArray [", Scalar (V, 0, 0), "] = ", Vector (VArray [V], 5, 1))
#end

//----------------------------------------------------

// Determines the Bezier patch in the 2D array that the function coordinates fall on
#macro ThisPatch (XVal, YVal)
	#debug "Starting macro ThisPatch \n"
    #local Mult = 1000;
    //#local _Patch = <0, 0>;
    #local _Patch = array [6];
    
    #for (U, 0, Usize)
    
    	#if (U = 0) #debug "U = ZERO \n" #end
    	//#local YVal=0;
    	//#local U=0;
    	//#local UArray[U]=<0,0,1.047>;
    
    
        #debug concat ("Testing YVal = ", Scalar (YVal, 8, 0), " vs \n [U=", Scalar (U, 8, 0), "] ", Vector (UArray [U], 32, 1), " \n")
        
        #if ( YVal >= UArray [U].y & YVal <= UArray [U].z )
        //#if ( int(YVal*Mult) >= int(UArray [U].y*Mult) & int(YVal*Mult) <= int(UArray [U].z*Mult) )

        #local _Patch [0] = UArray [U].x;
        #local _Patch [1] = UArray [U].y;
        #local _Patch [2] = UArray [U].z;
        #end 
    #end
    
    #ifndef (_Patch [0]) #debug "_Patch [0] U not defined \n" #end
    
    #for (V, 0, Vsize)
    #debug concat ("Testing X = ", Scalar (XVal, 3, 0), " vs ", Vector (VArray [V], 3, 0), " \n")
        #if (XVal >= VArray [V].y & XVal <= VArray [V].z)

        #local _Patch [3] = VArray [V].x;
        #local _Patch [4] = VArray [V].y;
        #local _Patch [5] = VArray [V].z;
        #end 
    #end
    
    #ifndef (_Patch [3]) #debug "_Patch [3] V not defined \n" #end
    #debug "macro ThisPatch done \n"
    _Patch
#end

//-----------------------------------------------------------
#macro Vector4 (_Value, _P, _N)
	#local _Result = concat ("<", vstr(4, _Value, ", ", 0, _P), ">");
	#if(_N) #local _Result = concat (_Result, " \n") #end
	_Result
#end
//-----------------------------------------------------------

// Determines where on individual Bezier patch function coordinates are
#macro PatchUV (XVal, YVal)
	#debug "Starting macro PatchUV \n"
    #local Range = ThisPatch (XVal, YVal);
    #local ThisU = (YVal - Range [1]) / (Range [2]- Range [1]);
    #local ThisV = (XVal - Range [4]) / (Range [5]- Range [4]);
    #local _Result = <Range [3], Range [0], ThisV, ThisU>;
    //#debug Vector4 (_Result, 3, 1)
	_Result
    
#end
//-----------------------------------------------------------

// Create an array for the Corner control points
#declare PatchArray = array [4][4];

//#declare Torus = union {
#for (U, 0, UU)
	#for (V, 0, VV)
		#declare Phi = (U*Phi_inc);
		#declare Theta = (V*Theta_inc);

		#local X1 = X (Theta,               Phi, r0, r1);
		#local X2 = X (Theta,               Phi + Phi_inc, r0, r1);
		#local X3 = X (Theta + Theta_inc,   Phi, r0, r1); 
		#local X4 = X (Theta + Theta_inc,   Phi + Phi_inc, r0, r1);
		
	
		#local Y1 = Y (Theta,               Phi, r0, r1); 
		#local Y2 = Y (Theta,               Phi + Phi_inc, r0, r1);
		#local Y3 = Y (Theta + Theta_inc,   Phi, r0, r1);
		#local Y4 = Y (Theta + Theta_inc,   Phi + Phi_inc, r0, r1);
		
		
		#local Z1 = Z (Theta, Phi, r1, 0);
		#local Z2 = Z (Theta, Phi + Phi_inc, r1, 0);
		#local Z3 = Z (Theta, Phi, r1, 0);
		#local Z4 = Z (Theta, Phi + Phi_inc, r1, 0);
		
	    // Perturb the points with a noise function
		#local P1 = <X1, Y1, Z1> + D * f_noise3d (X1, Y1, Z1);
		#local P2 = <X2, Y2, Z2> + D * f_noise3d (X2, Y2, Z2);
		#local P3 = <X3, Y3, Z3> + D * f_noise3d (X3, Y3, Z3);
		#local P4 = <X4, Y4, Z4> + D * f_noise3d (X4, Y4, Z4);
		
		#local PatchArray[0][0] = P1;
		#local PatchArray[0][3] = P2;
		#local PatchArray[3][0] = P3;
		#local PatchArray[3][3] = P4;
		
		#if (ParametricPoints)
			sphere {P1 Radius texture {pigment {Green} finish {specular 0.2}} }
			sphere {P2 Radius texture {pigment {Green} finish {specular 0.2}} }
			sphere {P3 Radius texture {pigment {Green} finish {specular 0.2}} }
			sphere {P4 Radius texture {pigment {Green} finish {specular 0.2}} }
		#end
		
		#if (ParametricGrid)
			cone {P1, Radius*2, P2, Radius texture {pigment {White}   finish {specular 0.2}} }
			cone {P2, Radius*2, P4, Radius texture {pigment {Red}     finish {specular 0.2}} }
			cone {P4, Radius*2, P3, Radius texture {pigment {Magenta} finish {specular 0.2}} }
			cone {P3, Radius*2, P1, Radius texture {pigment {Yellow}  finish {specular 0.2}} }
		#end
		
		//#debug concat ("U = ", str(U, 0, 3), ",   V = ", str(V, 0, 3), "\n")
		#declare StitchedArray [V][U] = PatchArray;

	#end
#end



// Interpolate internal control points 3 x 4 per patch.

#for (U, 0, UU, UStep)
	#for (V, 0, VV, VStep)
		#local Current = StitchedArray [V][U];
		
		#local VTopLeft      = Interpolate (Current[3][0], StitchedArray [Cycle(V-1, VV)][U][0][0], Current[0][0]);
		#local VTopRight     = Interpolate (Current[3][3], StitchedArray [Cycle(V-1, VV)][U][0][3], Current[0][3]);
		#local VBottomLeft   = Interpolate (StitchedArray [Cycle(V+1, VV)][U][3][0], Current[0][0], Current[3][0]);
		#local VBottomRight  = Interpolate (StitchedArray [Cycle(V+1, VV)][U][3][3], Current[0][3], Current[3][3]);
		
		#local HTopLeft      = Interpolate (Current[0][3], StitchedArray [V][Cycle(U-1, UU)][0][0], Current[0][0]);
		#local HTopRight     = Interpolate (StitchedArray [V][Cycle(U+1, UU)][0][3], Current[0][0], Current[0][3]);
		#local HBottomLeft   = Interpolate (Current[3][3], StitchedArray [V][Cycle(U-1, UU)][3][0], Current[3][0]);
		#local HBottomRight  = Interpolate (StitchedArray [V][Cycle(U+1, UU)][3][3], Current[3][0], Current[3][3]);

		// Extrapolate intermediate control points based on corners of current and surrounding patches

		#local Current[0][1] = Current [0][0] + HTopLeft;
		#local Current[0][2] = Current [0][3] - HTopRight;
		#local Current[3][1] = Current [3][0] + HBottomLeft;
		#local Current[3][2] = Current [3][3] - HBottomRight;
		
		#local Current[1][0] = Current [0][0] + VTopLeft;
		#local Current[2][0] = Current [3][0] - VBottomLeft;
		#local Current[1][3] = Current [0][3] + VTopRight;
		#local Current[2][3] = Current [3][3] - VBottomRight;
		
		#local Current[1][1] = Current [0][0] + HTopLeft + VTopLeft;
		#local Current[1][2] = Current [0][3] - HTopRight + VTopRight;
		#local Current[2][1] = Current [3][0] + HBottomLeft - VBottomLeft;
		#local Current[2][2] = Current [3][3] - HBottomRight - VBottomRight;
		
		// Transfer the current Bezier control point grid to the larger array
		#declare StitchedArray [V][U] = Current;

		// for showing grid of controlpoints for each patch
		#for (IU, 0, 3)
			#for (IV, 0, 3)
				#if (Grid)
					sphere {Current[IV][IU] Radius*3 texture {pigment {White} finish {specular 0.4}} }
				#end
				
				#if(Grid & IV<3)
					cone {Current[IV][IU], Radius, Current[IV+1][IU], Radius texture {pigment {Brown} finish {specular 0.2}} }
				#end
				
				#if(Grid & IU<3)
					cone {Current[IV][IU], Radius, Current[IV][IU+1], Radius texture {pigment {Brown} finish {specular 0.2}} }
				#end
				
				#if (Splines & IV<2)
					#local LastPoint = <0, 0, 0>;
					#for (T, 0, 1, 0.1)
						#local CurrentPoint = Point(V, U, T, IV);
						sphere {CurrentPoint Radius pigment {Green} }
						#if (T > 0)
							cylinder {LastPoint, CurrentPoint, Radius pigment {Green} }
						#end
						#local LastPoint = CurrentPoint;
					#end // end for T
				#end
				
				#if (Splines & IU<2)
					#local LastPoint = <0, 0, 0>;
					#for (T, 0, 1, 0.1)
						#local CurrentPoint = Point(V, U, IU, T);
						sphere {CurrentPoint Radius pigment {Yellow} }
						#if (T > 0)
							cylinder {LastPoint, CurrentPoint, Radius pigment {Yellow} }
						#end
						#local LastPoint = CurrentPoint;
					#end // end for T
				#end
				
				
				
				
			#end // end for IV
		#end // end for IU
		
		
		// show the vectors connecting corner points on either side of control points
		#if (ShowIVectors)
			//V
		    Check (Current[3][0], StitchedArray [Cycle(V-1, VV)][U][0][0], White)
		    Check (Current[3][3], StitchedArray [Cycle(V-1, VV)][U][0][3], Red)
		    Check (StitchedArray [Cycle(V+1, VV)][U][3][0], Current[0][0], Yellow)
		    Check (StitchedArray [Cycle(V+1, VV)][U][3][3], Current[0][3], Magenta)

		    //H
		    Check (Current[0][3], StitchedArray [V][Cycle(U-1, UU)][0][0], White*0.2)
		    Check (Current[0][0], StitchedArray [V][Cycle(U+1, UU)][0][3], Red*0.2)
		    Check (StitchedArray [V][Cycle(U-1, UU)][3][0], Current[3][3], Yellow*0.2)
		    Check (StitchedArray [V][Cycle(U+1, UU)][3][3], Current[3][0], Magenta*0.2)
		#end

	#end // end for V
#end // end for U

/*
// output all control points
#for (U, 0, UU, UStep)
	#for (V, 0, VV, VStep)
		#for (IU, 0, 3)
			#for (IV, 0, 3)
				#debug concat (Scalar(V, 0, 0), ", ", Scalar(U, 0, 0), ", ", Scalar(IU, 0, 0), ", ", Scalar(IV, 0, 1), " = ", Vector (StitchedArray [V][U][IU][IV], 3, 1))
			#end
		#end
	#end
#end

#error "BREAK"
*/

//------------------------------------------------------------------------------

// Make bicubic patches
#declare Individual = false;
#declare UVtex = false;
#declare E = 5;

#declare Us = Usize-1; //UU;
#declare Vs = Vsize; //VV;

#if (Patches)
union {
#for (U, 0, UU, UStep)
		#for (V, 0, VV, VStep)		
		
		#if (Corners)
			sphere {StitchedArray [V][U][0][0] Radius*E texture {pigment {White} finish {specular 0.2}} }
			sphere {StitchedArray [V][U][0][3] Radius*E texture {pigment {Red} finish {specular 0.2}} }
			sphere {StitchedArray [V][U][3][0] Radius*E texture {pigment {Yellow} finish {specular 0.2}} }
			sphere {StitchedArray [V][U][3][3] Radius*E texture {pigment {Magenta} finish {specular 0.2}} }
		#end
		
		#if (Squares)
			// Top Left
			ShowSquares (
				StitchedArray [Cycle(V-1, VV)][Cycle(U-1, UU)][2][2],
				StitchedArray [Cycle(V-1, VV)][U][2][1],
				StitchedArray [V][Cycle(U-1, UU)][1][2],
				StitchedArray [V][U][1][1],
				//StitchedArray [V][U][0][0],
				//StitchedArray [V][U][1][0],
				//StitchedArray [V][U][0][1],
				//StitchedArray [V][U][1][1],
				SquareTexture
				)
		#end
		
		bicubic_patch {
		   	type 1 
		   	flatness 0.001
		   	u_steps BT 
		   	v_steps BT  //  uses 2^u_steps * 2^v_steps triangles to create patch
		   	
			#if (Individual)
				uv_vectors <0,0> <1,0> <1,1> <0,1>
			#else
				// DEFINE THIS PATCH'S POSITION IN THE GRID
   				// Calculate proper uv vectors for unified mapping
   				// torus: the map is taken from the area <0,0> <1,1> where the u-coordinate is wrapped 
   				// 	around the major radius and the the v-coordinate is wrapped around the minor radius. 
   				
   				// Note: Recent additions revealed torus mapping to be reversed with respect to the u-coordinate. 
   				// 	For backward compatibility reasons it remains the same in this release, however this may change in the future.
   				// These go around clockwise from top left to bottom left
   				uv_vectors 
				< V*(1/Vs),     U*(1/Us)> 
		   		<(V+1)*(1/Vs),  U*(1/Us)> 
		   		<(V+1)*(1/Vs),  (U+1)*(1/Us)> 
		   		< V*(1/Vs),     (U+1)*(1/Us)> // define as part of stiched-array whole


   				//#debug concat ("UU = ", str(UU, 0, 3), ",   VV = ", str(VV, 0, 3), "\n")
   				//#debug concat ("U = ", str(U, 0, 3), ",   V = ", str(V, 0, 3), "\n")
   				//#debug concat ("U*(1/Us) = ", str(U*(1/Us), 0, 3), ",   (V*(1/Vs) = ", str(V*(1/Vs), 0, 3), "\n")
			#end

		   	// Specify control points for the bicubic patch object
		   	#for (IU, 0, 3)
				#for (IV, 0, 3)
					#if (IU*IV < 9)
					StitchedArray [V][U][IV][IU],
					#else
					StitchedArray [V][U][IV][IU]
					#end
				#end
			#end
		 
			uv_mapping

			#if (Individual)
			 	texture {pigment {checker 
			 			color rgbf <1, 1, 1, 0.1> 
		         		color rgbf <0, 0, 1, 0.7> 
		         		scale 1/3
		      			} // end pigment pattern
		      			finish {phong 0.6 phong_size 20}
				} // end texture
		   	
			   	interior_texture {pigment {checker 
		         		color rgbf <1, 1, 1, 0.1> 
		         		color rgbf <0, 0.5, 0, 0.7> 
		         		scale 1/3
		      			} // end pigment pattern
		      			finish {phong 0.6 phong_size 20}
		   		} // end interior_texture 
	   		#else
	   			//texture  { pigment { image_map { png "Numbered_Grid-Red-Blue.png"  } scale 1} rotate x*0 finish {phong 0.6 phong_size 20}}
	   			texture {UVTexture}
				//texture  { pigment {Black} }	   			
			#end
		} // end bicubic patch
	
	#end // end for V
#end // end for U

// This only rotates the patches, not the rest - rotate the camera for whole scene
//rotate y*90
//rotate -z*90
//rotate -x*45
//rotate y*90
//translate -y*5

} // end union
#end // end if Patches

#if (Curve)

    #declare N = 5;
	#declare _X = function (_N, Angle, _M) {min (_N*mod(Angle, 1/_N) * _M, _M)}
	#declare _Y = function (Angle, _M) {min(max(0, Angle * _M), _M)}
	
	
	#declare F = 1.0000001;	// LastPoint multiplier to avoid [most] degenerate cylinders
	#for (Theta, 0, 1, 0.001)
	
		#debug concat ("_X = ", Scalar (_X(N, Theta, Vsize), 5, 0), ", _Y = ", Scalar (_Y(Theta, Usize), 5, 1))
		
		#local UV = PatchUV ( _X(N, Theta, Vsize), _Y(Theta, Usize) );
		// this converts the points to 'flat' UV, but now they need to be converted to the interpolated Bezier coordinates		
		#local CurrentPoint = Point (UV.x, UV.y, UV.z, UV.t);
		//#debug Vector (Point, 3, 1)
		//#debug Vector (UV, 3, 1)
		sphere {CurrentPoint Radius pigment {White*Theta}}
		#if (Theta > 0)
			cylinder {LastPoint*F, CurrentPoint, Radius texture {CurveTex} }
		#end
		#local LastPoint = CurrentPoint;
	#end

#end

/*
	#declare T = 0;
	#declare F = 1.0000001;
	#declare N = 5;
	#for (U, U1, U2-Phi_inc, Phi_inc/CurveStep)
		#for (V, V1, V2-Theta_inc, Theta_inc/CurveStep)
			//CurveStep defined at beginning of file
			#local LastPoint = <0, 0, 0>;
			#local PS  = U*Phi_inc;			#local TS  = V*Theta_inc;
			#local PE  = (U+1)*Phi_inc;		#local TE  = (V+1)*Theta_inc;
			#local PR  = PE - PS;			#local TR  = TE - TS;
			#local PRS = PR/CurveStep;		#local TRS = TR/CurveStep;
				
			#for (P, PS, PE, PRS)
				#for (T, TS, TE, TRS)
					#local CurveV = (P-PS)/PR;  // converts span to 0-1 range
					#local CurveU = (T-TS)/TR;
			//
			#for (T, 0, 1, 0.001)
			#local CurveV = N * mod (T, 1/N);
			#local CurveU = T;
			#local CurrentPoint = Point(V, U, CurveV, CurveU);
			sphere {CurrentPoint Radius*4 texture {CurveTex} }
			#if (T > 0)
				cylinder {LastPoint*F, CurrentPoint, Radius*4 texture {CurveTex} }
			#end
			#local LastPoint = CurrentPoint;
			
				//#end // end for P
			#end // end for T
		#end // end for V
	#end // end for U
#end // end if Curve
*/

