#version 3.7;
global_settings {assumed_gamma 1.0}

#declare SDL = true;
#include "colors.inc"
#include "BillsArrayMacros.pov"

#declare Feet = 12;

camera { 
    location  <0.5, 0.5, -1.1>
    look_at   <0.5, 0.5, 0>
    right x*image_width/image_height
    up y
  } 


light_source {<0, 0, -5> color White}

#declare UStart = 0;
#declare UEnd = 1;
#declare UDim = 18;
#declare UStep = (UEnd-UStart)/UDim;

#declare VStart = 0;
#declare VEnd = 1;
#declare VDim = 12;
#declare VStep = (VEnd-VStart)/VDim;

#declare UArray = array [UDim];
#declare VArray = array [VDim];

#declare USize = dimension_size (UArray, 1);
#declare VSize = dimension_size (VArray, 1);

#declare UU = USize -1;
#declare VV = VSize -1;

#declare Width  = 1;
#declare Height = 1;
#declare Line  = 1/(20*max(USize, VSize));
#declare Line2 = 1/(10*max(USize, VSize));


// draw grid
#for (X, 0, 1, 1/VSize)
	cylinder {<X, 0, 0>, <X, 1, 0> Line pigment {Green}}
#end

#for (Y, 0, 1, 1/USize)
	cylinder {<0, Y, 0>, <1, Y, 0> Line pigment {Red}}
#end


#for (U, 0, UU)
    #declare UArray [U] = <U, U*UStep, (U+1)*UStep>;
#end

#for (V, 0, VV)
    #declare VArray [V] = <V, V*VStep, (V+1)*VStep>;
#end

// Determines which Bezier patch in array the function coordinates fall on
#macro ThisPatch (XVal, YVal)

    //#local _Patch = <0, 0>;
    #local _Patch = array [6];
    
    #for (U, 0, UU)
        //#debug concat ("Testing ", Scalar (YVal, 3, 0), " vs ", Vector (UArray [U], 3, 0), " \n")
        #if (YVal >= UArray [U].y & YVal <= UArray [U].z)
        //#local _Patch = _Patch + <UArray [U].x, 0, 0>;
        #local _Patch [0] = UArray [U].x;
        #local _Patch [1] = UArray [U].y;
        #local _Patch [2] = UArray [U].z;
        #end 
    #end
    
    #ifndef (_Patch [0]) #debug "_Patch [0] U not defined \n" #end
    
    #for (V, 0, VV)
        #if (XVal >= VArray [V].y & XVal <= VArray [V].z)
        //#local _Patch = _Patch + <0, VArray [V].x, 0>;
        #local _Patch [3] = VArray [V].x;
        #local _Patch [4] = VArray [V].y;
        #local _Patch [5] = VArray [V].z;
        #end 
    #end
    
    _Patch
#end

// Determines where on individual Bezier patch function coordinates are
#macro PatchUV (XVal, YVal)
    #local Range = ThisPatch (XVal, YVal);
    #local ThisU = (YVal - Range [1]) / (Range [2]- Range [1]);
    #local ThisV = (XVal - Range [4]) / (Range [5]- Range [4]);
    <ThisV, ThisU, 0>
#end


// Draw diagonal line function
#declare _X = function (N, Angle) {N*mod(Angle, 1/N)}
#declare _Y = function (Angle) {Angle}

#declare N=5;
#for (Theta, 0, 1, 0.001)
	#local Point = <_X(N, Theta), _Y(Theta), 0>;
	#local UV = PatchUV ( _X(N, Theta), _Y(Theta) );
	//#debug Vector (Point, 3, 1)
	//#debug Vector (UV, 3, 1)
	sphere {Point Line2 pigment {White*Theta}}
#end
